@@ -56,6 +56,7 @@ FTPGetOneF(
 	char *src, *srclim;
 	char *dst, *dstlim;
 	char outbuf[512];
+	int cr;
 #endif
 	longest_int tstartPoint;
 	volatile longest_int startPoint = 0;
@@ -97,6 +98,10 @@ FTPGetOneF(
 		 *
 		 */
 
+		if ((file == NULL) || (file[0] == '\0') || (dstfile == NULL) || (dstfile[0] == '\0')) {
+			return (kErrBadParameter);
+		}
+
 		AutomaticallyUseASCIIModeDependingOnExtension(cip, file, &xtype);
 
 		if ((cip->progress != (FTPProgressMeterProc) 0) || (resumeflag == kResumeYes) || (resumeProc != kNoFTPConfirmResumeDownloadProc)) {
@@ -399,6 +404,7 @@ FTPGetOneF(
 
 #if ASCII_TRANSLATION
 	if (xtype == kTypeAscii) {
+		cr = 0;
 		/* Ascii */
 		for (;;) {
 			if (! WaitForRemoteInput(cip)) {	/* could set cancelXfer */
@@ -472,10 +478,6 @@ FTPGetOneF(
 			dst = outbuf;
 			dstlim = dst + sizeof(outbuf);
 			while (src < srclim) {
-				if (*src == '\r') {
-					src++;
-					continue;
-				}
 				if (dst >= dstlim) {
 					nwrote = write(fd, outbuf, (write_size_t) (dst - outbuf));
 					if (nwrote == (write_return_t) (dst - outbuf)) {
@@ -495,6 +497,89 @@ FTPGetOneF(
 						goto brk;
 					}
 				}
+
+				/*
+				 * The FTP protocol says we are to receive
+				 * CR+LF as the end-of-line (EOLN) sequence for
+				 * all files sent in ASCII mode.
+				 *
+				 * However, many servers only convert from
+				 * their native EOLN format to CR+LF.
+				 * For example, a Windows ISP being used
+				 * to host Mac OS 9 text files (whose EOLN
+				 * is just a CR) could violate protocol
+				 * and send the file with the raw CRs only.
+				 * It should be converting the file from CR
+				 * to CR+LF for us, but many do not, so we
+				 * try to convert raw CRs to our EOLN.
+				 *
+				 * We also look for raw LFs and convert them
+				 * if needed.
+				 */
+				if (cr != 0) {
+					/* Leftover CR from previous buffer */
+					cr = 0;
+					goto have_char_after_cr;
+				}
+
+				if (*src == '\n') {
+					/* protocol violation: raw LF */
+					src++;
+					goto add_eoln;
+				}
+				if (*src == '\r') {
+					src++;
+					if (src < srclim) {
+have_char_after_cr:
+						if (*src == '\n') {
+							/* Normal case:
+							 * CR+LF eoln.
+							 */
+							src++;
+						} else if (*src == '\r') {
+							/* Try to work-around
+							 * server bug which
+							 * sends CR+CR+LF
+							 * eolns.
+							 */
+							if (((src + 1) < srclim) && (src[1] == '\n'))  {
+								src += 2;
+							}
+						}
+						/* else {protocol violation: raw CR} */
+					} else {
+						cr = 1;
+						continue;
+					}
+add_eoln:
+					if ((dst + 2) >= dstlim) {
+						/* Write out buffer before
+						 * adding one or two chars.
+						 */
+						nwrote = write(fd, outbuf, (write_size_t) (dst - outbuf));
+						if (nwrote == (write_return_t) (dst - outbuf)) {
+							/* Success. */
+							dst = outbuf;
+						} else if (errno == EPIPE) {
+							result = kErrWriteFailed;
+							cip->errNo = kErrWriteFailed;
+							errno = EPIPE;
+							(void) shutdown(cip->dataSocket, 2);
+							goto brk;
+						} else {
+							FTPLogError(cip, kDoPerror, "Local write failed.\n");
+							result = kErrWriteFailed;
+							cip->errNo = kErrWriteFailed;
+							(void) shutdown(cip->dataSocket, 2);
+							goto brk;
+						}
+					}
+					*dst++ = cip->textEOLN[0];
+					if (cip->textEOLN[1] != '\0')
+						*dst++ = cip->textEOLN[1];
+					continue;
+				}
+
 				*dst++ = *src++;
 			}
 			if (dst > outbuf) {
@@ -522,6 +607,30 @@ FTPGetOneF(
 			cip->bytesTransferred += (longest_int) nread;
 			FTPUpdateIOTimer(cip);
 		}
+		if (cr != 0) {
+			/* Last block ended with a raw CR.
+			 * Write out one last end-of-line.
+			 */
+			nwrote = write(fd, cip->textEOLN, (write_size_t) strlen(cip->textEOLN));
+			if (nwrote == (write_return_t) strlen(cip->textEOLN)) {
+				/* Success. */
+				if (mdtm != kModTimeUnknown) {
+					(void) utime(dstfile, &ut);
+				}
+			} else if (errno == EPIPE) {
+				result = kErrWriteFailed;
+				cip->errNo = kErrWriteFailed;
+				errno = EPIPE;
+				(void) shutdown(cip->dataSocket, 2);
+				goto brk;
+			} else {
+				FTPLogError(cip, kDoPerror, "Local write failed.\n");
+				result = kErrWriteFailed;
+				cip->errNo = kErrWriteFailed;
+				(void) shutdown(cip->dataSocket, 2);
+				goto brk;
+			}
+		}
 	} else
 #endif	/* ASCII_TRANSLATION */
 	{
@@ -647,7 +756,7 @@ brk:
 	(void) signal(SIGPIPE, (FTPSigProc) osigpipe);
 #endif	/* NO_SIGNALS */
 
-	if ((mdtm != kModTimeUnknown) && (cip->bytesTransferred > 0)) {
+	if (mdtm != kModTimeUnknown) {
 		(void) utime(dstfile, &ut);
 	}
 
