@@ -1,6 +1,6 @@
 /* bookmark.h
  *
- * Copyright (c) 1992-2000 by Mike Gleason.
+ * Copyright (c) 1992-2003 by Mike Gleason.
  * All rights reserved.
  * 
  */
@@ -31,7 +31,7 @@ typedef struct Bookmark {
 
 #define kBookmarkVersion		8
 #define kBookmarkMinVersion		3
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	define kBookmarkFileName		"bookmarks.txt"
 #else
 #	define kBookmarkFileName		"bookmarks"
@@ -40,7 +40,7 @@ typedef struct Bookmark {
 #define kOldBookmarkFileName		"hosts"
 #define kBookmarkBupFileName		"bookmarks.old"
 
-#define BMTINDEX(p) ((int) ((char *) p - (char *) gBookmarkTable) / sizeof(Bookmark))
+#define BMTINDEX(p) ((int) ((char *) p - (char *) gBookmarkTable) / (int) sizeof(Bookmark))
 
 /* bookmark.c */
 void BookmarkToURL(BookmarkPtr, char *, size_t);
@@ -57,3 +57,4 @@ int LoadBookmarkTable(void);
 BookmarkPtr SearchBookmarkTable(const char *);
 void SortBookmarks(void);
 void DefaultBookmarkName(char *, size_t, char *);
+void DisposeBookmarkTable(void);
