@@ -12,7 +12,7 @@
 #	pragma hdrstop
 #endif
 
-#if defined(WIN32) || defined(_WINDOWS)
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	include "..\ncftp\util.h"
 #	include "..\ncftp\spool.h"
 #	include "..\ncftp\pref.h"
@@ -91,8 +91,8 @@ Usage(void)
   ncftpget -a -d /tmp/debug.log -t 60 ftp.wustl.edu . '/pub/README*'\n");
 
 	(void) fprintf(fp, "\nLibrary version: %s.\n", gLibNcFTPVersion + 5);
-	(void) fprintf(fp, "\nThis is a freeware program by Mike Gleason (mgleason@ncftp.com).\n");
-	(void) fprintf(fp, "This was built using LibNcFTP (http://www.ncftp.com/libncftp).\n");
+	(void) fprintf(fp, "\nThis is a freeware program by Mike Gleason (http://www.ncftp.com).\n");
+	(void) fprintf(fp, "This was built using LibNcFTP (http://www.ncftp.com/libncftp/).\n");
 
 	ClosePager(fp);
 	DisposeWinsock();
@@ -171,8 +171,8 @@ main(int argc, char **argv)
 	char urlfile[256];
 	char urldir[512];
 	int urlxtype;
-	LineList cdlist;
-	LinePtr lp;
+	FTPLineList cdlist;
+	FTPLinePtr lp;
 	int rc;
 	int nD = 0;
 	int batchmode = 0;
@@ -221,6 +221,25 @@ main(int argc, char **argv)
 	postcmd[0] = '\0';
 	perfilecmd[0] = '\0';
 
+	GetoptReset(&opt);
+	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:t:aRTr:vVf:ADzZEFbcB:W:X:Y:")) > 0) {
+		if (c == 'b') {
+			batchmode++;
+		}
+	}
+
+	if (batchmode > 0) {
+		GetoptReset(&opt);
+		while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:U:t:mar:RvVf:AT:S:EFcyZzDbB:W:X:Y:")) > 0) switch(c) {
+			case 'v': case 'V': case 'A': case 'B': case 'T':
+			case 'd': case 'e': case 't': case 'r': case 'c':
+			case 'z': case 'Z':
+				(void) fprintf(stderr, "The \"-%c\" option is not valid when used with conjunction with \"-%c\".\n", c, 'b');
+				exit(kExitUsage);
+				break;
+		}
+	}
+
 	GetoptReset(&opt);
 	while ((c = Getopt(&opt, argc, argv, "P:u:j:p:e:d:t:aRTr:vVf:ADzZEFbcB:W:X:Y:")) > 0) switch(c) {
 		case 'P':
@@ -302,7 +321,7 @@ main(int argc, char **argv)
 			fi.dataPortMode = kPassiveMode;
 			break;
 		case 'b':
-			batchmode++;
+			/* handled above */
 			break;
 		case 'B':
 			fi.dataSocketRBufSize = (size_t) atol(opt.arg);	
@@ -328,9 +347,6 @@ main(int argc, char **argv)
 	if (opt.ind > argc - 1)
 		Usage();
 
-	if ((ftpcat != 0) && (batchmode != 0))
-		Usage();
-
 	if (progmeters != 0)
 		fi.progress = PrStatBar;
 
@@ -549,8 +565,14 @@ main(int argc, char **argv)
 			}
 		} else {
 			if ((rc = FTPGetFiles3(&fi, urlfile, ".", rflag, kGlobYes, xtype, resumeflag, appendflag, deleteflag, tarflag, kNoFTPConfirmResumeDownloadProc, 0)) < 0) {
-				FTPPerror(&fi, rc, kErrCouldNotStartDataTransfer, "ncftpget", NULL);
-				es = kExitXferFailed;
+				if (rc == kErrLocalSameAsRemote) {
+					/* Show the message, but do not err-out. */
+					FTPPerror(&fi, rc, kErrCouldNotStartDataTransfer, "ncftpget", NULL);
+					es = kExitSuccess;
+				} else {
+					FTPPerror(&fi, rc, kErrCouldNotStartDataTransfer, "ncftpget", NULL);
+					es = kExitXferFailed;
+				}
 			} else {
 				es = kExitSuccess;
 
