@@ -39,7 +39,7 @@ SSendto(int sfd, const char *const buf, size_t size, int fl, const struct sockad
 
 	time(&now);
 	done = now + tlen;
-	tleft = (done < now) ? ((alarm_time_t) (done - now)) : 0;
+	tleft = (done > now) ? ((alarm_time_t) (done - now)) : 0;
 	forever {
 		(void) alarm(tleft);
 		nwrote = sendto(sfd, buf, (send_size_t) size, fl,
@@ -52,7 +52,7 @@ SSendto(int sfd, const char *const buf, size_t size, int fl, const struct sockad
 			break;		/* Fatal error. */
 		errno = 0;
 		time(&now);
-		tleft = (done < now) ? ((alarm_time_t) (done - now)) : 0;
+		tleft = (done > now) ? ((alarm_time_t) (done - now)) : 0;
 		if (tleft < 1) {
 			nwrote = kTimeoutErr;
 			errno = ETIMEDOUT;
@@ -88,7 +88,7 @@ SSendto(int sfd, const char *const buf, size_t size, int fl, const struct sockad
 				SETWSATIMEOUTERR
 				return (kTimeoutErr);
 			}
-			tleft = (int) (done - now);
+			tleft = (done > now) ? ((int) (done - now)) : 0;
 			errno = 0;
 			MY_FD_ZERO(&ss);
 #if defined(__DECC) || defined(__DECCXX)
